Method: RubyLabs::RecursionLab#start_group
- Defined in:
- lib/recursionlab.rb
#start_group(a, i, j) ⇒ Object
Method called at the start of each call to merge – position the progress bar below the group and initialize the auxilliary region where merged groups are put.
363 364 365 366 367 |
# File 'lib/recursionlab.rb', line 363 def start_group(a,i,j) @@drawing.groupstart = i @@drawing.group = [] a.set_region(i, j, @@drawing., @@drawing.) end |